//More PCF programming in Java



import com.dmitri.pcf.*
import com.ibm.mq.pcf.*;
import com.ibm.mq.*;

public class TestClass {

/**
*   For testing only.
*/
public static void main(String[] args) {

try {
    PCFAgent iAgent = new PCFAgent("localhost", 1414, 
                                   "SYSTEM.DEF.SVRCONN");

    EnquireQueuePCF.enableQueueGet("SYSTEM.DEFAULT.LOCAL.QUEUE", 
                                   iAgent);
    iAgent.disconnect();

  } catch (Exception ex) {
    ex.printStackTrace();
  }
}
}


package com.dmitri.pcf;
import com.ibm.mq.pcf.*;
import com.ibm.mq.*;

/**
*   @author Dmitri
*
*  This class is used to start channels. The class uses various 
*  responses, including static and dynamic ones, and ones that 
*  are human-oriented and machine-oriented.
*/
public class StartChannelPCF implements CMQCFC {

  private PCFAgent iAgent;

/**
*  Constructor, this requires a valid queue manager agent.
*/
  public StartChannelPCF(PCFAgent agent) {

    iAgent = agent;
  }

/**
*  This method starts the channel specified. The result is returned 
*  in a format that is suitable for displaying to users.
*/
  public String startChannelString(String channelName) {

    return startChannelString(channelName, iAgent);
  }

/**
*  This method starts the channel specified. The result is returned 
*  as an integer value for further processing.
*/
  public int startChannel(String channelName) {

    return startChannel(channelName, iAgent);
  }

/**
*  This method starts the channel specified. The result is returned 
*  in a format suitable for displaying to users. Static version.
*/
  public static String startChannelString(String channelName, 
                                          PCFAgent agent) {

    int reason = startChannel(channelName, agent);
    if (reason == 0) {
      return("Channel " + channelName + " Started");
    }
    return("Channel " + channelName + 
           " failed to start, reason code : " + reason);
  }

/**
*  This method starts the channel specified. The result is returned 
*  as an integer for further processing. Static version, actual 
*  call implemented here.
*/
  public static int startChannel(String channelName, PCFAgent agent) {

    PCFParameter [] parameters = new PCFParameter [] {
            new MQCFST (MQCACH_CHANNEL_NAME, channelName),
            };

    PCFHashtable response = null;

    try {
      MQMessage [] pcfResponses = agent.send (MQCMD_START_CHANNEL, 
                                              parameters);
      response = new PCFHashtable(pcfResponses[0]);
                                      // assume only the one response.

      if (response.isValid()) {
        return 0;
      }

    } catch (Exception e) {
      e.printStackTrace();
    }
    return response.getReasonCode();
  }
}


package com.dmitri.pcf;
import com.ibm.mq.pcf.*;
import com.ibm.mq.*;

/**
*   @author Dmitri
*
*  This class is used to stop channels. Various methods are used 
*  to achieve this.
*/
public class StopChannelPCF implements CMQCFC {

  private PCFAgent iAgent;

/**
*  Constructor. This requires a valid queue manager agent (from 
*  the com.ibm.mq.pcf package).
*/
  public StopChannelPCF(PCFAgent agent) {

    iAgent = agent;
  }

/**
*  This method stops the channel specified. The result is returned
*  in plain text that may be displayed for an operator to read.
*/
  public String stopChannelString(String channelName) {
    return stopChannelString(channelName, iAgent);
  }

/**
*  This method stops the channel specified. The result is returned
*  in plain text that may be displayed for an operator to read.
*  Static version
*/
  public static String stopChannelString(String channelName, 
                                         PCFAgent agent) {

    int reason = stopChannel(channelName, agent);

    if (reason == 0) {
      return("Channel " + channelName + " Stopped Successfully");
    }
    return("Channel " + channelName + 
           " failed to stop, reason code : " + reason);
  }

/**
*  This method stops the channel specified. The result is returned 
*  in code format suitable for further processing.
*/
  public int stopChannel(String channelName) {

    return stopChannel(channelName, iAgent);
  }

/**
*  This method stops the channel specified. The result is returned 
*  in code format suitable for further processing. An alternative 
*  static version actually makes the call.
*/
  public static int stopChannel(String channelName, PCFAgent agent) {

    PCFParameter [] parameters = new PCFParameter [] {
            new MQCFST (MQCACH_CHANNEL_NAME, channelName),
            new MQCFIN (MQIACF_QUIESCE, MQQO_NO ),
                                      // always force channel closed.
            };

    PCFHashtable response = null;

    try {
      MQMessage [] pcfResponses = agent.send (MQCMD_STOP_CHANNEL, 
                                              parameters);
      response = new PCFHashtable(pcfResponses[0]);
                                     // assume only the one response.

      if (response.isValid()) {
        return 0;
      }

    } catch (Exception e) {
      e.printStackTrace();
    }
    return response.getReasonCode();
  }
}


package com.dmitri.pcf;
import com.ibm.mq.pcf.*;
import com.ibm.mq.*;

/**
*   @author Dmitri
*
*  This class is used to reset channels. Again, various methods 
*  are provided.
*/
public class ResetChannelPCF implements CMQCFC {

  private PCFAgent iAgent;

/**
*  Constructor, this requires a valid queue manager agent.
*/
  public ResetChannelPCF(PCFAgent agent) {

    iAgent = agent;
  }

/**
*  This method resets the channel's message sequence number to 0, 
*  which means that the next message will be message 1. The result 
*  is returned in plain text that may be read by an operator.
*/
  public String resetChannelString(String channelName) {

    return resetChannelString(channelName, iAgent);
  }

/**
*  This method resets the channel's message sequence number to 0, 
*  which means that the next message will be message 1. The result 
*  is returned as an integer for further processing.
*/
  public int resetChannel(String channelName) {

    return resetChannel(channelName, iAgent);
  }

/**
*  This method resets the channel's message sequence number to 0, 
*  which means that the next message will be message 1. The result 
*  is returned in plain text that may be read by an operator.
*  Static version.
*/
  public static String resetChannelString(String channelName, 
                                          PCFAgent agent) {

    int reason = resetChannel(channelName, agent);

    if (reason == 0) {
      return("Channel " + channelName + " Reset");
    }
    return("Channel " + channelName + 
           " failed to reset, reason code : " + reason );
  }

/**
*  This method resets the channel's message sequence number to 0, 
*  which means that the next message will be message 1. While MQSeries 
*  allows any number to be specified as the new MSN, I prefer just to 
*  reset both sides of a channel to 0. This is the static method 
*  that actually makes the calls.
*/
  public static int resetChannel(String channelName, PCFAgent agent) {

    PCFParameter [] parameters = new PCFParameter [] {
            new MQCFST (MQCACH_CHANNEL_NAME, channelName),
            };

    PCFHashtable response = null;

    try {
      MQMessage [] pcfResponses = agent.send (MQCMD_RESET_CHANNEL, 
                                              parameters);
      response = new PCFHashtable(pcfResponses[0]);
                                     // assume only the one response.

      if (response.isValid()) {
        return 0;
      }

    } catch (Exception e) {
      e.printStackTrace();
    }
    return response.getReasonCode();
  }
}


package com.dmitri.pcf;
import com.ibm.mq.pcf.*;
import com.ibm.mq.*;

/**
*   @author Dmitri
*
*  This class is used to ping channels. The ping function may be 
*  used to check communications to channels. A pingable channel is 
*  not necessarily suitable for the transmission of messages, 
*  though, as other factors, such as sequence numbers, may prevent 
*  transmission.
*/
public class PingChannelPCF implements CMQCFC {

  private PCFAgent iAgent;

/**
*  Constructor, this requires a valid queue manager agent.
*/
  public PingChannelPCF(PCFAgent agent) {

    iAgent = agent;
  }

/**
*  This method pings the channel specified. The result returned 
*  by this method is suitable for further processing.
*/
  public int pingChannel(String channelName) {

    return pingChannel(channelName, iAgent);
  }

/**
*  This method pings the channel specified. The string returned 
*  contains a general description of the outcome. This is in plain 
*  text and is aimed at humans, not other processes.
*/
  public String pingChannelString(String channelName) {

    return pingChannelString(channelName, iAgent);
  }

/**
*  This method pings the channel specified. The string returned 
*  contains a general description of the outcome. This is in plain 
*  text and is aimed at humans, not other processes. Static version.
*/
  public static String pingChannelString(String channelName, 
                                         PCFAgent agent) {

    int reason = pingChannel(channelName, agent);

    if (reason == 0) {
      return("Channel " + channelName + " Pinged Sucessfully");
    }
    return("Channel " + channelName + 
           " failed to ping, reason code : " + reason);
  }

/**
*  This static method pings the channel specified. The result is 
*  suitable for further processing.
*/
  public static int pingChannel(String channelName, PCFAgent agent) {

    PCFParameter [] iParameters = new PCFParameter [] {
            new MQCFST (MQCACH_CHANNEL_NAME, channelName),
            };

    PCFHashtable response = null;

    try {
      MQMessage [] pcfResponses = agent.send (MQCMD_PING_CHANNEL, 
                                              iParameters);
      response = new PCFHashtable(pcfResponses[0]);
                                   // assume only the one response.

      if (response.isValid()) {
        return 0;
      }

    } catch (Exception e) {
      e.printStackTrace();
    }
    return response.getReasonCode();
  }
}


package com.dmitri.pcf;
import com.ibm.mq.pcf.*;
import com.ibm.mq.*;

/**
*   @author Dmitri
*
*  This class is used to query queue information. Queue depth is 
*  of particular interest when the required queue is a transmission 
*  queue (xmitq). In addition to querying, it is also possible to 
*  re-enable (GET enable) queues using this class. This is useful 
*  when developing systems management software, as channels often 
*  disable their transmission queues when problems are encountered.
*/
public class EnquireQueuePCF implements CMQCFC, CMQC {

  private PCFAgent iAgent;

/**
*  The constructor. This requires a valid queue manager agent. It may 
*  then be used to query any queue belonging to the queue manager.
*/
  public EnquireQueuePCF(PCFAgent agent) {

    iAgent = agent;
  }

/**
*  This method returns details of the queue in question.
*/
  public PCFHashtable queueDetails(String queueName) {

    return queueDetails(queueName, iAgent);
  }

/**
*  This method extracts the queue depth from the supplied queue 
*  details.
*/
  public static int getQueueDepth(PCFHashtable details) {

    return details.getIntValue(MQIA_CURRENT_Q_DEPTH);
  }

/**
*  This method returns details of all queues belonging to the current 
*  queue manager.
*/
  public PCFHashtable [] getAllQueues() {

    return getAllQueues(iAgent);
  }

/**
*  GET-enable a queue. The return value is zero for success, 
*  positive for MQSeries errors, and negative for general errors.
*/
  public int enableQueueGet(String queueName) {
    return enableQueueGet(queueName, iAgent);
  }

/**
*  Re-enable queue triggering. The return value is zero for success, 
*  positive for MQSeries errors, and negative for general errors.
*/
  public int enableQueueTriggering(String queueName) {
    return enableQueueTriggering(queueName, iAgent);
  }

/**
*  GET-enable a queue. This method may be used as a template for 
*  other queue modification methods. The method returns zero if 
*  successful.
*/
  public static int enableQueueGet(String queueName, PCFAgent agent) {

    try {

      // Check that it's appropriate to re-enable the queue.

      PCFHashtable details = queueDetails(queueName, agent);
      int queueType = details.getIntValue(MQIA_Q_TYPE);
      if (queueType != MQQT_ALIAS && queueType != MQQT_LOCAL) {
        return -1;  // not valid for remote queues etc.
      }

      if (details.getIntValue(MQIA_INHIBIT_GET) == MQQA_GET_ALLOWED) {
        return 0;  // nothing further to do, it's already GET-enabled.
      }

      PCFParameter [] parameters = new PCFParameter [] {
              new MQCFST (MQCA_Q_NAME, queueName),
              new MQCFIN (MQIA_Q_TYPE, queueType),
              new MQCFIN (MQIA_INHIBIT_GET, MQQA_GET_ALLOWED),
              };

      MQMessage [] pcfResponses = agent.send (MQCMD_CHANGE_Q, 
                                              parameters);
      PCFHashtable response = new PCFHashtable(pcfResponses[0]);
                                    // assume only the one response.
      if (response.isValid()) {
        return 0;  // successful call.
      } else {
        return response.getReasonCode();
      }

    } catch (Exception ex) {
      ex.printStackTrace();
    }
    return -1;
  }

/**
*  Re-enables queue triggering. Returns zero if successful.
*/
  public static int enableQueueTriggering(String queueName, 
                                          PCFAgent agent) {

    try {

      // Check that it's appropriate to re-enable the queue.

      PCFHashtable details = queueDetails(queueName, agent);
      int queueType = details.getIntValue(MQIA_Q_TYPE);
      if (queueType != MQQT_LOCAL) {
        return -1;  // only valid for local queues.
      }

      if (details.getIntValue(MQIA_TRIGGER_CONTROL) == MQTC_ON) {
        return 0;
               // nothing further to do, it's already trigger-enabled.
      }

      PCFParameter [] parameters = new PCFParameter [] {
              new MQCFST (MQCA_Q_NAME, queueName),
              new MQCFIN (MQIA_Q_TYPE, queueType),
              new MQCFIN (MQIA_TRIGGER_CONTROL, MQTC_ON),
              };

      MQMessage [] pcfResponses = agent.send (MQCMD_CHANGE_Q, 
                                              parameters);
      PCFHashtable response = new PCFHashtable(pcfResponses[0]);
                                    // assume only the one response.
      if (response.isValid()) {
        return 0;  // successful call.
      } else {
        return response.getReasonCode();
      }

    } catch (Exception ex) {
      ex.printStackTrace();
    }
    return -1;
  }

/**
*   An alternative static queue enquiry method.
*/
  public static PCFHashtable queueDetails(String queueName, 
                                          PCFAgent agent) {

    PCFParameter [] parameters = new PCFParameter [] {
            new MQCFST (MQCA_Q_NAME, queueName),
            };

    try {
      MQMessage [] pcfResponses = agent.send (MQCMD_INQUIRE_Q, 
                                              parameters);

      PCFHashtable response = new PCFHashtable(pcfResponses[0]);
                                    // assume only the one response.
      return response;

    } catch (Exception e) {
      e.printStackTrace();
    }
    return null;
  }

/**
*  Alternative static version of the getAllQueues method.
*/
  public static PCFHashtable [] getAllQueues(PCFAgent agent) {

    try {
      PCFParameter [] parameters = new PCFParameter [] {
              new MQCFST (MQCA_Q_NAME, "*"),
              };

      MQMessage [] pcfResponses = agent.send (MQCMD_INQUIRE_Q, 
                                              parameters);
      PCFHashtable [] details = new PCFHashtable [pcfResponses.length];

      for (int i = 0; i < pcfResponses.length; i++) {
        details[i] = new PCFHashtable(pcfResponses[i]);
      }
      return details;

    } catch (Exception e) {
      e.printStackTrace();
    }
    return null;
  }
}


package com.dmitri.pcf;

import com.ibm.mq.pcf.*;
import com.ibm.mq.*;
import java.util.*;
/**
*  @author Dmitri
*
*  This class extends Hashtable to provide PCF-specific processing.
*/
public class PCFHashtable extends Hashtable {

  private int iReasonCode;

/**
*  Constructor that sets up the data based on a message.
*/
  PCFHashtable(MQMessage message) {

    super();
    try {
      MQCFH cfh = new MQCFH(message);
      iReasonCode = cfh.reason;

      if (isValid()) {

        PCFParameter p;
        for (int i = 0; i < cfh.parameterCount; i++) {

          // Walk through the returned attributes
          p = PCFParameter.nextParameter (message);
          Integer key = new Integer(p.getParameter());
          put(key, p.getValue());
        }
      }
    } catch (Exception ex) {
      System.out.println(ex);
    }
  }

/**
*  Is the PCF request valid and, hence, may it be used?
*/
  public boolean isValid() {
    return iReasonCode == 0;
  }

/**
*  Returns the reason code for use by diagnostic processes.
*/
  public int getReasonCode(){
    return iReasonCode;
  }

/**
*  Returns the int value represented by the key supplied.
*/
  public int getIntValue(int key) {
    Integer i = (Integer) get(new Integer(key));
    return i.intValue();
  }

/**
*  Returns the int array represented by the key supplied.
*/
  public int [] getIntArray(int key) {
    return (int []) get(new Integer(key));
  }

/**
*  Returns the string value represented by the key supplied.
*/
  public String getStringValue(int key) {
    // mq returns padded strings.
    return ((String) get(new Integer(key))).trim();
  }

/**
*  Returns the string array represented by the key supplied.
*/
  public String [] getStringArray(int key) {
    String [] paddedStrings =  (String []) get(new Integer(key));
    String [] trimStrings = new String [paddedStrings.length];
    for (int i = 0; i < paddedStrings.length; i++) {
      trimStrings[i] = paddedStrings[i].trim();
    }
    return trimStrings;
  }
}



